@@ -4,8 +4,8 @@
 # @Filename:    pythonian.py
 # @Author:      Samuel Hill
 # @Date:        2020-02-10 16:10:26
-# @Last Modified by:   Samuel Hill
-# @Last Modified time: 2020-03-02 16:02:12
+# @Last Modified by:    Samuel Hill
+# @Last Modified time:  2020-09-11 05:00:08
 
 """Pythonian agent, sits on top of the modified KQMLModule -
 CompanionsKQMLModule. Uses subscription management classes to allow for cleaner
@@ -60,7 +60,7 @@ class Pythonian(CompanionsKQMLModule):
         self.asks = {}
         self.subscriptions = SubscriptionManager()
         self.polling_interval = 1
-        self.poller = Thread(target=self._poll_for_subscription_updates,
+        self.poller = Thread(target=self.poll_for_subscription_updates,
                              args=[])
         LOGGER.info('Starting subcription poller...')
         self.poller.start()
@@ -82,7 +82,7 @@ class Pythonian(CompanionsKQMLModule):
             msg (KQMLPerformative): overall message to be passed along in reply
             content (KQMLList): tell content from companions to be logged
         """
-        LOGGER.debug('received tell: %s', content)
+        LOGGER.info('received tell: %s', content)
         reply_msg = performative(f'(tell :sender {self.name} :content :ok)')
         self.reply(msg, reply_msg)
 
@@ -117,7 +117,7 @@ class Pythonian(CompanionsKQMLModule):
         (car) is then used to find the function bound to the ask predicate, and
         that function is called with the bounded argument list unpacked into
         it's own inputs. The resulting query is then passed along to the
-        _response_to_query helper which will properly respond to patterns or
+        response_to_query helper which will properly respond to patterns or
         bindings based on out response type.
 
         Arguments:
@@ -130,7 +130,7 @@ class Pythonian(CompanionsKQMLModule):
         """
         if content.head() not in self.asks:
             error_msg = f'No ask query predicate named {content.head()} known'
-            LOGGER.debug(error_msg)
+            LOGGER.warning(error_msg)
             self.error_reply(msg, error_msg)
             return
         bounded = []
@@ -142,19 +142,19 @@ class Pythonian(CompanionsKQMLModule):
         if expected_args != len(bounded):
             error_msg = (f'Expected {expected_args} input arguments to query '
                          f'predicate {content.head()}, got {len(bounded)}')
-            LOGGER.debug(error_msg)
+            LOGGER.warning(error_msg)
             self.error_reply(msg, error_msg)
             return
         LOGGER.info('received ask-one %s', content.head())
         try:
             results = self.asks[content.head()](*bounded)
         except (TypeError, ValueError) as except_msg:
-            LOGGER.debug('Failed execution: %s, %s', except_msg, print_exc())
+            LOGGER.warning('Failed execution: %s, %s', except_msg, print_exc())
             error_msg = f'An error occurred while executing: {content.head()}'
             self.error_reply(msg, error_msg)
             return
         LOGGER.debug('Ask-one returned results: %s', results)
-        self._response_to_query(msg, content, results, msg.get('response'))
+        self.response_to_query(msg, content, results, msg.get('response'))
 
     ###########################################################################
     #                            Achieve Functions                            #
@@ -162,6 +162,8 @@ class Pythonian(CompanionsKQMLModule):
 
     def achieve_on_agent(self, receiver: str, data: Any):
         """Sends a KQML achieve to the receiver with the data input as a list.
+        The data is passed through listify to assign it to the proper KQML
+        types so keep this in mind when passing alond the achieve.
 
         Arguments:
             receiver (str): name of the receiving agent
@@ -169,8 +171,6 @@ class Pythonian(CompanionsKQMLModule):
         """
         msg = performative(f'(achieve :sender {self.name} :receiver {receiver}'
                            f' :content {listify(data)})')
-        # TODO: Do we want to listify the data for an achieve or rely on
-        # proper formatting?
         self.send(msg)
 
     def add_achieve(self, func: Callable[..., Any], name: str = None):
@@ -210,18 +210,18 @@ class Pythonian(CompanionsKQMLModule):
         if content.head() != 'task':
             error_msg = (f'Only support achieve command of task, instead got '
                          f'{content.head()}')
-            LOGGER.debug(error_msg)
+            LOGGER.warning(error_msg)
             self.error_reply(msg, error_msg)
             return
         action = content.get('action')
         if not action:
             error_msg = 'No action for achieve task provided'
-            LOGGER.debug(error_msg)
+            LOGGER.warning(error_msg)
             self.error_reply(msg, error_msg)
             return
         if action.head() not in self.achieves:
             error_msg = f'No action named {action.head()} is known'
-            LOGGER.debug(error_msg)
+            LOGGER.warning(error_msg)
             self.error_reply(msg, error_msg)
             return
         achieve_question = self.achieves[action.head()]
@@ -230,14 +230,14 @@ class Pythonian(CompanionsKQMLModule):
         if expected_args != len(actual_args):
             error_msg = (f'Expected {expected_args} input arguments to achieve'
                          f' task {action.head()}, got {len(actual_args)}')
-            LOGGER.debug(error_msg)
+            LOGGER.warning(error_msg)
             self.error_reply(msg, error_msg)
             return
         LOGGER.info('received achieve %s', action.head())
         try:
             results = self.achieves[action.head()](*actual_args)
         except (TypeError, ValueError) as except_msg:
-            LOGGER.debug('Failed execution: %s, %s', except_msg, print_exc())
+            LOGGER.warning('Failed execution: %s, %s', except_msg, print_exc())
             error_msg = f'An error occurred while executing {action.head()}'
             self.error_reply(msg, error_msg)
             return
@@ -340,19 +340,19 @@ class Pythonian(CompanionsKQMLModule):
         if content.head() != 'ask-all':
             error_msg = (f'Only supports ask-all subscription, received '
                          f'unsupported performative {content.head()}')
-            LOGGER.debug(error_msg)
+            LOGGER.warning(error_msg)
             self.error_reply(msg, error_msg)
             return
         query = content.get('content')
         pattern = query.to_string()
         if query.head() not in self.asks:
             error_msg = f'No ask named {query.head()} is known'
-            LOGGER.debug(error_msg)
+            LOGGER.warning(error_msg)
             self.error_reply(msg, error_msg)
             return
         if pattern not in self.subscriptions:
             error_msg = f'Ask ({query.head()}) is not subscribable'
-            LOGGER.debug(error_msg)
+            LOGGER.warning(error_msg)
             self.error_reply(msg, error_msg)
             return
         LOGGER.info('received subscription %s to %s', msg, pattern)
@@ -360,22 +360,32 @@ class Pythonian(CompanionsKQMLModule):
         reply_msg = f'(tell :sender {self.name} :content :ok)'
         self.reply(msg, performative(reply_msg))
 
-    def _poll_for_subscription_updates(self):
+    def poll_for_subscription_updates(self):
         """Goes through the subscription updates as they come in and properly
         respond to the query."""
         while self.ready:
             for _, subscription in self.subscriptions.items():
                 if subscription.new_data is not None:
-                    LOGGER.info('updating subscriptions for %s', subscription)
+                    LOGGER.debug('updating subscriptions for %s', subscription)
                     for subscriber in subscription:
                         ask = subscriber.get('content')
                         query = ask.get('content')
-                        self._response_to_query(subscriber, query,
-                                                subscription.new_data,
-                                                ask.get('response'))
+                        self.response_to_query(subscriber, query,
+                                               subscription.new_data,
+                                               ask.get('response'))
                     subscription.retire_data()
             sleep(self.polling_interval)
 
+    def exit(self, n: int = 0):
+        """Override of companionsKQMLModule exit, calls super().exit(n) and
+        then joins the polling Thread.
+
+        Args:
+            n (int, optional): the value to pass along to sys.exit
+        """
+        super().exit(n)
+        self.poller.join()
+
     ###########################################################################
     #                             Insert Functions                            #
     ###########################################################################
@@ -475,7 +485,7 @@ class Subscription():
 
     Attributes:
         new_data (Any): new data to be used in updating the subscription query
-            pattern (passed along to _response_to_query).
+            pattern (passed along to response_to_query).
         old_data (Any): copy of the new data after it has been retired, used
             for only updating new if it differs from the last value.
         subscribers (list): list of subscription messages to reply to when
@@ -513,7 +523,7 @@ class Subscription():
 
         Args:
             data (Any): new data to be used in updating the subscription query
-                pattern (passed along to _response_to_query)
+                pattern (passed along to response_to_query)
         """
         if self.old_data != data:
             self.new_data = data
